/*  osdchat - on-screen display messaging service
 *  Copyright (c) 2003 by Mike Smelov  <mike-at-argoncorp-dot-com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  See LICENSE for details.
 *
 */

 /* config.c */

#include "osdchatd.h"
#include "common.h"
#include "version.h"

/* defined in common.c */
extern char *prog_name;

/* default user preference xosd values */
struct xosd_s xset = {
	.offset = 80,
	.delay = 5,
	.lines = 5,
	.pos = XOSD_bottom,
	.color = DEFAULT_XOSD_COLOR,
	.font = DEFAULT_XOSD_COLOR,
};

/* default user preferences */
struct pref_s pref = {
	.serv_port = SERV_PORT,
	.wrapcol = 60,
	.max_clients = 10,
	.move = DOWN,
	.max_warnlevel = 25,
	.display = ":0.0",
	.p_format = "\\u@\\h(\\y)>\\s",		/* "user@host(tty)> " */
	.logmsgs = 0,
	.timestamp = 0,
	.loud_connect = 0,
	.loud_disconnect = 1,
	.loud_block = 0,
	.blocklist = 0,
};

/* print program usage information */
void print_usage()
{
	printf(	"osdchat daemon " OC_VERSION "\n"
		"Copyright (c) 2003 by Mike Smelov <http://osdchat.xexix.net>\n\n"
		"usage:	%s [options]\n\n"
		"options:\n"
		"\t-a <n>\t\tSelect broadcast announcements\n"
		"\t-b\t\tBlock clients that have exceeded warning level\n"
		"\t-c <color>\tSet color (see rgb.txt)  (Default:  red)\n"
		"\t-d <display>\tX Display string (env. variable DISPLAY)\n"
		"\t\t\t(Default:  \":0.0\")\n"
		"\t-D\t\tTurn debugging mode on -- use only for testing\n "
		"\t-f <font>\tSet font (use xfontsel for font choices)\n"
		"\t-h\t\tShow this usage information\n"
		"\t-k\t\tKill currently running daemon\n"
		"\t-l <lines>\tSet number of lines to show on OSD  (Default:  5)\n"
		"\t-L\t\tLog messages to ~/.osdchat.log\n"
		"\t-m <clients>\tSet maximum number of clients  (Default:  10)\n"
		"\t-o <offset>\tSet offset number of pixels  (Default:  80)\n"
		"\t-p <port>\tPort # to listen on  (Default:  7447)\n"
		"\t-P <format>\tCustomize prompt format\n"
		"\t-s <seconds>\tSeconds before OSD hides  (Default:  5)\n"
		"\t-t\t\tShow text on top\n"
		"\t-T\t\tShow timestamps\n"
		"\t-u\t\tText moves upward\n"
		"\t-v\t\tPrint version\n"
		"\t-w <width>\tWrap text at given width  (Default:  60)\n"
		"\t-W <level>\tSet maximum warning level  (Default:  25) \n\n", prog_name);

	fflush(stdout);
	exit(EXIT_SUCCESS);
}

/* Check entry in the config table.
   Returns index to which option was set.
   Returns -1 if no option is found.
 */
int find_config_opt(struct config_set *entry)
{
	int i;

	/* d's are set to default in parse_config */
	struct config_set table[CONFIG_NOPTS] = {
		{ "Announcements", 	"-a", 1 },
		{ "BlockList",		"-b", 0 },	/* d */
		{ "Color", 		"-c", 1 },
		{ "Display", 		"-d", 1 },
		{ "Font", 		"-f", 1 },
		{ "MaxLines", 		"-l", 1 },
		{ "LogMessages",	"-L", 0 },	/* d */
		{ "MaxClients", 	"-m", 1 },
		{ "Offset", 		"-o", 1 },
		{ "Port",		"-p", 1 },
		{ "PromptFormat", 	"-P", 1 },
		{ "ShowDelay", 		"-s", 1 },
		{ "OnTop", 		"-t", 0 },	/* d */
		{ "TimeStamp", 		"-T", 0 },	/* d */
		{ "MoveUp", 		"-u", 0 },	/* d */
		{ "LineWidth",		"-w", 1 },
		{ "MaxWarnLevel",	"-W", 1 },
		{ NULL, 		NULL, 0 }
	};

	for (i = 0; table[i].opt != NULL; i++) {
		if ( strncmp(entry->opt, table[i].opt, 32) == 0 ) {
			entry->arg = table[i].arg;
			entry->narg = table[i].narg;
			return i;
		}
	}

	return -1;
}

/* parse user + system configuration file */
void parse_config()
{
	static char *arg_v[2*CONFIG_NOPTS] = {0};
	int used[CONFIG_NOPTS] = {0};
	
	char loc_path[BUFSIZE];
	char buf[BUFSIZE];
	char item[32], val[128];
	struct config_set entry;
	FILE *f = NULL;
	int i, arg_c;

	if (user_info->pw_dir != NULL) {
		snprintf(loc_path, sizeof(loc_path), "%s/%s", user_info->pw_dir, CONFIG_USER);

		printDBG(LVL_DEBUG, "Opening user config file: %s", loc_path);
		if ( (f = fopen(loc_path, "r")) == NULL ) {
			printDBG(LVL_DEBUG, "...not found\n");
		}
	}

	if (!f) {
		printDBG(LVL_DEBUG, "Opening system config file: " CONFIG_SYS);
		if ( (f = fopen(CONFIG_SYS, "r")) == NULL ) {
			printDBG(LVL_DEBUG, "...not found\n");
			return;
		}
	}
	printDBG(LVL_DEBUG, "\n");

	if (arg_v[0] != prog_name)
		arg_v[0] = prog_name;

	arg_c = 1;

	/* set these to defaults */
	xset.pos = XOSD_bottom;
	pref.timestamp = 0;
	pref.move = DOWN;
	pref.blocklist = 0;
	pref.logmsgs = 0;

	/* scan config file line by line */
	while(fgets(buf, BUFSIZE, f)) {
		/* ignore comments + empty lines*/
		if (buf [0] == '#' || buf[0] == '\r' || buf[0] == '\n')
			continue;

		sscanf(buf, "%31s %127s", item, val);

		entry.opt = item;
		if ((i = find_config_opt(&entry)) != -1 && !used[i]) {
			used[i] = 1;
			if ( (entry.narg == 1 && val[0] != NIL) || entry.narg == 0 ) {
				if (arg_v[arg_c] == NULL)
					arg_v[arg_c] = (char *) xmalloc(sizeof(val));

				strncpy(arg_v[arg_c++], entry.arg, sizeof(val));

				if (entry.narg == 0)
					continue;

				if (arg_v[arg_c] == NULL)
					arg_v[arg_c] = (char *) xmalloc(sizeof(val));

				strncpy(arg_v[arg_c++], val, sizeof(val));
			}
		}

		bzero(val, strlen(val));
	}

	(void) fclose(f);
	arg_v[arg_c] = NULL;

	parse_args(arg_c, arg_v);
}

/* parse command line options */
void parse_args(int argc, char **argv)
{
	int i, opt;
	struct servent *sp;

	if (argc < 2)
		return;

	printDBG(LVL_DEBUG, "Parsing switches\n");

	optind = 0;
	opterr = 1;
	while ((opt = getopt(argc, argv, "a:bc:d:Df:l:Lm:o:p:P:s:tTuvw:W:")) != -1) {
		switch (opt) {
		case 'a':
			i = atoi(optarg);
			set_limits(&i, 0 , 7);
			pref.loud_connect = (i & 1) ? 1 : 0;
			pref.loud_disconnect = (i & 2) ? 1 : 0;
			pref.loud_block = (i & 4) ? 1 : 0;
			break;
		case 'b':
			pref.blocklist = 1;
			cs_block_init();
			break;
		case 'c':
			xset.color = optarg;
			break;
		case 'd':
			pref.display = optarg;
			break;
		case 'D':
			break;	/* handled in main */
		case 'f':
			xset.font = optarg;
			break;
		case 'l':
			xset.lines = atoi(optarg);
			set_limits(&xset.lines, 1, MAX_LINES);
			break;
		case 'L':
			pref.logmsgs = 1;
			break;
		case 'm':
			pref.max_clients = atoi(optarg);
			set_limits(&pref.max_clients, 1, MAX_NCLIENTS);
			break;
		case 'o':
			xset.offset = atoi(optarg);
			set_limits(&xset.offset, -1000, 1000);
			break;
		case 'p':
			if ( (pref.serv_port = atoi(optarg)) == 0 ) {
				if ( (sp = getservbyname(optarg, "tcp")) == NULL) {
					printDBG(LVL_SYSLOG, "getservbyname() error for: %s", optarg);
					pref.serv_port = SERV_PORT;
				} else {
					pref.serv_port = ntohs(sp->s_port);
				}
			}
			set_limits(&pref.serv_port, 0, 65535);
			printDBG(LVL_DEBUG, "Listing on port %d\n", pref.serv_port);
			break;
		case 'P':
			pref.p_format = optarg;
			break;
		case 's':
			xset.delay = atoi(optarg);
			set_limits(&xset.delay, 0, 60*MAX_DELAY);
			break;
		case 't':
			xset.pos = XOSD_top;
			break;
		case 'T':
			pref.timestamp = 1;
			break;
		case 'u':
			pref.move = UP;
			break;
		case 'v':
			printf(OC_VERSION "\n");
			break;
		case 'w':
			pref.wrapcol = atoi(optarg);
			set_limits(&pref.wrapcol, MIN_LINECHAR, MAX_LINECHAR);
			break;
		case 'W':
			pref.max_warnlevel = atoi(optarg);
			set_limits(&pref.max_warnlevel, 1, MAX_WARNLEVEL);
			break;
		case '?':
			print_usage(argv[0]);
		}
	}

	/* set X display to output to */
	printDBG(LVL_DEBUG, "Setting Display = %s\n", pref.display);
	if ( setenv("DISPLAY", pref.display, 1) < 0 ) {
		printDBG(LVL_SYSLOG, "%s", strerror(errno));
		exit(EXIT_FAILURE);
	}

	if (debug == 1) {
		printDBG(LVL_DEBUG, "arg_v[0-%d] = ", argc - 1);
		for (i = 0; i < argc; i++)
			printDBG(LVL_DEBUG, "%s ", argv[i]);
		printDBG(LVL_DEBUG, "\n");
	}
}
